#ifndef VERSION_H_IN
#define VERSION_H_IN

#include <QDateTime>
#include <QGuiApplication>
#include <QScreen>
#include <QString>
#include <QSysInfo>

namespace across {
inline static const QString GUI_VERSION() {
    QString version("0.1.2");

    return version;
}

inline static const QString BUILD_INFO() {
    QString buildInfo("Arch Linux - 0.1.3");
    return buildInfo;
}

inline static const QString BUILD_TIME() {
    QDateTime time = QDateTime::currentDateTimeUtc();
    return time.toString();
}

inline static const QString EXTRA_INFO() {
    QSysInfo sys_info;
    QScreen *screen = QGuiApplication::screens().at(0);

    QString info =
        QString("OS: %1 | Kernel: %2 | Platform: %3 | Screen: %4x%5 | DPI: %6")
            .arg(sys_info.productType(), sys_info.kernelVersion(),
                 QGuiApplication::platformName())
            .arg(screen->size().rwidth())
            .arg(screen->size().rheight())
            .arg(screen->logicalDotsPerInch());

    return info;
}

inline static const QString SOURCE_CODE_URL() {
    return QString("https://github.com/ArkToria/ACross");
}

inline static const QString ISSUES_URL() {
    return QString("https://github.com/ArkToria/ACross/issues");
}

inline static const QString LICENSE_URL() {
    return QString("https://www.gnu.org/licenses/gpl-3.0.html");
}

inline static const QString API_URL(uint per_page = 0) {
    if (per_page == 0) {
        return QString(
            "https://api.github.com/repos/ArkToria/ACross/releases/latest");
    } else {
        return QString("https://api.github.com/repos/ArkToria/ACross/"
                       "releases?per_page=%1")
            .arg(per_page);
    }
}

inline static const QString RELEASE_URL() {
    return QString("https://github.com/ArkToria/ACross/releases");
}

inline static const QStringList FONT_LIST() {
    QString fonts_fallback("Noto Sans, Segoe UI Variable, Segoe UI, Noto Sans CJK, PingFang SC, Microsoft YaHei UI");
    const QStringList default_fonts = {
        "Noto Sans",     "Segoe UI Variable", "Segoe UI",
        "Noto Sans CJK", "PingFang SC",       "Microsoft YaHei UI",
    };

    do {
        if (fonts_fallback.isEmpty())
            break;

        QStringList temp_fonts;
        for (auto &font : fonts_fallback.split(",")) {
            temp_fonts.append(font.trimmed());
        }

        if (temp_fonts.isEmpty())
            break;

        return temp_fonts;
    } while (false);

    return default_fonts;
}

inline static const QString DATE_TIME_FORMAT() {
    QString format_string("MM/dd/yyyy hh:mm:ss");

    if (format_string.isEmpty())
        format_string = QString("MM/dd/yyyy hh:mm:ss");

    return format_string;
}
} // namespace across

#endif // VERSION_H_IN
